@@ -51,7 +51,7 @@ static int64_t mz_stream_ioapi_tell(void *stream);
 static int32_t mz_stream_ioapi_seek(void *stream, int64_t offset, int32_t origin);
 static int32_t mz_stream_ioapi_close(void *stream);
 static int32_t mz_stream_ioapi_error(void *stream);
-static void *mz_stream_ioapi_create(void **stream);
+static void *mz_stream_ioapi_create(void);
 static void mz_stream_ioapi_delete(void **stream);
 
 /***************************************************************************/
@@ -121,7 +121,7 @@ static int32_t mz_stream_ioapi_read(void *stream, void *buf, int32_t size) {
     } else
         return MZ_PARAM_ERROR;
 
-    return zread(opaque, ioapi->handle, buf, size);
+    return (int32_t)zread(opaque, ioapi->handle, buf, size);
 }
 
 static int32_t mz_stream_ioapi_write(void *stream, const void *buf, int32_t size) {
@@ -142,7 +142,7 @@ static int32_t mz_stream_ioapi_write(void *stream, const void *buf, int32_t size
     } else
         return MZ_PARAM_ERROR;
 
-    written = zwrite(opaque, ioapi->handle, buf, size);
+    written = (int32_t)zwrite(opaque, ioapi->handle, buf, size);
     return written;
 }
 
@@ -233,15 +233,10 @@ static int32_t mz_stream_ioapi_set_filefunc64(void *stream, zlib_filefunc64_def
     return MZ_OK;
 }
 
-static void *mz_stream_ioapi_create(void **stream) {
-    mz_stream_ioapi *ioapi = NULL;
-
-    ioapi = (mz_stream_ioapi *)calloc(1, sizeof(mz_stream_ioapi));
+static void *mz_stream_ioapi_create(void) {
+    mz_stream_ioapi *ioapi = (mz_stream_ioapi *)calloc(1, sizeof(mz_stream_ioapi));
     if (ioapi)
         ioapi->stream.vtbl = &mz_stream_ioapi_vtbl;
-    if (stream)
-        *stream = ioapi;
-
     return ioapi;
 }
 
@@ -333,17 +328,20 @@ zipFile zipOpen2(const char *path, int append, const char **globalcomment,
 
     if (pzlib_filefunc_def) {
         if (pzlib_filefunc_def->zopen_file) {
-            if (!mz_stream_ioapi_create(&stream))
+            stream = mz_stream_ioapi_create();
+            if (!stream)
                 return NULL;
             mz_stream_ioapi_set_filefunc(stream, pzlib_filefunc_def);
         } else if (pzlib_filefunc_def->opaque) {
-            if (!mz_stream_create(&stream, (mz_stream_vtbl *)pzlib_filefunc_def->opaque))
+            stream = mz_stream_create((mz_stream_vtbl *)pzlib_filefunc_def->opaque);
+            if (!stream)
                 return NULL;
         }
     }
 
     if (!stream) {
-        if (!mz_stream_os_create(&stream))
+        stream = mz_stream_os_create();
+        if (!stream)
             return NULL;
     }
 
@@ -370,17 +368,20 @@ zipFile zipOpen2_64(const void *path, int append, const char **globalcomment,
 
     if (pzlib_filefunc_def) {
         if (pzlib_filefunc_def->zopen64_file) {
-            if (!mz_stream_ioapi_create(&stream))
+            stream = mz_stream_ioapi_create();
+            if (!stream)
                 return NULL;
             mz_stream_ioapi_set_filefunc64(stream, pzlib_filefunc_def);
         } else if (pzlib_filefunc_def->opaque) {
-            if (!mz_stream_create(&stream, (mz_stream_vtbl *)pzlib_filefunc_def->opaque))
+            stream = mz_stream_create((mz_stream_vtbl *)pzlib_filefunc_def->opaque);
+            if (!stream)
                 return NULL;
         }
     }
 
     if (!stream) {
-        if (!mz_stream_os_create(&stream))
+        stream = mz_stream_os_create();
+        if (!stream)
             return NULL;
     }
 
@@ -405,7 +406,10 @@ zipFile zipOpen_MZ(void *stream, int append, const char **globalcomment) {
     int32_t mode = zipConvertAppendToStreamMode(append);
     void *handle = NULL;
 
-    mz_zip_create(&handle);
+    handle = mz_zip_create();
+    if (!handle)
+        return NULL;
+
     err = mz_zip_open(handle, stream, mode);
 
     if (err != MZ_OK) {
@@ -459,6 +463,12 @@ int zipOpenNewFileInZip5(zipFile file, const char *filename, const zip_fileinfo
     if (!compat)
         return ZIP_PARAMERROR;
 
+    /* The filename and comment length must fit in 16 bits. */
+    if (filename && strlen(filename) > 0xffff)
+        return ZIP_PARAMERROR;
+    if (comment && strlen(comment) > 0xffff)
+        return ZIP_PARAMERROR;
+
     memset(&file_info, 0, sizeof(file_info));
 
     if (zipfi) {
@@ -470,8 +480,8 @@ int zipOpenNewFileInZip5(zipFile file, const char *filename, const zip_fileinfo
             dos_date = mz_zip_tm_to_dosdate(&zipfi->tmz_date);
 
         file_info.modified_date = mz_zip_dosdate_to_time_t(dos_date);
-        file_info.external_fa = zipfi->external_fa;
-        file_info.internal_fa = zipfi->internal_fa;
+        file_info.external_fa = (uint32_t)zipfi->external_fa;
+        file_info.internal_fa = (uint16_t)zipfi->internal_fa;
     }
 
     if (!filename)
@@ -534,7 +544,7 @@ int zipOpenNewFileInZip3_64(zipFile file, const char *filename, const zip_filein
     const void *extrafield_local, uint16_t size_extrafield_local, const void *extrafield_global,
     uint16_t size_extrafield_global, const char *comment, int compression_method, int level,
     int raw, int windowBits, int memLevel, int strategy, const char *password,
-    uint32_t crc_for_crypting, int zip64) {
+    unsigned long crc_for_crypting, int zip64) {
     return zipOpenNewFileInZip4_64(file, filename, zipfi, extrafield_local, size_extrafield_local,
         extrafield_global, size_extrafield_global, comment, compression_method, level, raw, windowBits,
         memLevel, strategy, password, crc_for_crypting, MZ_VERSION_MADEBY, 0, zip64);
@@ -565,6 +575,14 @@ int zipOpenNewFileInZip(zipFile file, const char *filename, const zip_fileinfo *
         extrafield_global, size_extrafield_global, comment, compression_method, level, 0);
 }
 
+int zipOpenNewFileInZip64(zipFile file, const char *filename, const zip_fileinfo *zipfi,
+    const void *extrafield_local, uint16_t size_extrafield_local, const void *extrafield_global,
+    uint16_t size_extrafield_global, const char *comment, int compression_method, int level,
+    int zip64) {
+    return zipOpenNewFileInZip2_64(file, filename, zipfi, extrafield_local, size_extrafield_local,
+        extrafield_global, size_extrafield_global, comment, compression_method, level, 0, zip64);
+}
+
 int zipOpenNewFileInZip_64(zipFile file, const char *filename, const zip_fileinfo *zipfi,
     const void *extrafield_local, uint16_t size_extrafield_local, const void *extrafield_global,
     uint16_t size_extrafield_global, const char *comment, int compression_method, int level,
@@ -592,7 +610,7 @@ int zipCloseFileInZipRaw64(zipFile file, uint64_t uncompressed_size, unsigned lo
     mz_compat *compat = (mz_compat *)file;
     if (!compat)
         return ZIP_PARAMERROR;
-    return mz_zip_entry_close_raw(compat->handle, (int64_t)uncompressed_size, crc32);
+    return mz_zip_entry_close_raw(compat->handle, (int64_t)uncompressed_size, (uint32_t)crc32);
 }
 
 int zipCloseFileInZip(zipFile file) {
@@ -672,17 +690,20 @@ unzFile unzOpen2(const char *path, zlib_filefunc_def *pzlib_filefunc_def) {
 
     if (pzlib_filefunc_def) {
         if (pzlib_filefunc_def->zopen_file) {
-            if (!mz_stream_ioapi_create(&stream))
+            stream = mz_stream_ioapi_create();
+            if (!stream)
                 return NULL;
             mz_stream_ioapi_set_filefunc(stream, pzlib_filefunc_def);
         } else if (pzlib_filefunc_def->opaque) {
-            if (!mz_stream_create(&stream, (mz_stream_vtbl *)pzlib_filefunc_def->opaque))
+            stream = mz_stream_create((mz_stream_vtbl *)pzlib_filefunc_def->opaque);
+            if (!stream)
                 return NULL;
         }
     }
 
     if (!stream) {
-        if (!mz_stream_os_create(&stream))
+        stream = mz_stream_os_create();
+        if (!stream)
             return NULL;
     }
 
@@ -706,17 +727,20 @@ unzFile unzOpen2_64(const void *path, zlib_filefunc64_def *pzlib_filefunc_def) {
 
     if (pzlib_filefunc_def) {
         if (pzlib_filefunc_def->zopen64_file) {
-            if (!mz_stream_ioapi_create(&stream))
+            stream = mz_stream_ioapi_create();
+            if (!stream)
                 return NULL;
             mz_stream_ioapi_set_filefunc64(stream, pzlib_filefunc_def);
         } else if (pzlib_filefunc_def->opaque) {
-            if (!mz_stream_create(&stream, (mz_stream_vtbl *)pzlib_filefunc_def->opaque))
+            stream = mz_stream_create((mz_stream_vtbl *)pzlib_filefunc_def->opaque);
+            if (!stream)
                 return NULL;
         }
     }
 
     if (!stream) {
-        if (!mz_stream_os_create(&stream))
+        stream = mz_stream_os_create();
+        if (!stream)
             return NULL;
     }
 
@@ -753,9 +777,11 @@ unzFile unzOpen_MZ(void *stream) {
     int32_t err = MZ_OK;
     void *handle = NULL;
 
-    mz_zip_create(&handle);
-    err = mz_zip_open(handle, stream, MZ_OPEN_MODE_READ);
+    handle = mz_zip_create();
+    if (!handle)
+        return NULL;
 
+    err = mz_zip_open(handle, stream, MZ_OPEN_MODE_READ);
     if (err != MZ_OK) {
         mz_zip_delete(&handle);
         return NULL;
@@ -1081,12 +1107,56 @@ int unzGoToNextFile(unzFile file) {
     return err;
 }
 
-int unzLocateFile(unzFile file, const char *filename, unzFileNameComparer filename_compare_func) {
+#if !defined(MZ_COMPAT_VERSION) || MZ_COMPAT_VERSION < 110
+#ifdef WIN32
+#  define UNZ_DEFAULT_IGNORE_CASE 1
+#else
+#  define UNZ_DEFAULT_IGNORE_CASE 0
+#endif
+
+int unzLocateFile(unzFile file, const char *filename, unzFileNameCase filename_case) {
     mz_compat *compat = (mz_compat *)file;
     mz_zip_file *file_info = NULL;
     uint64_t preserve_index = 0;
     int32_t err = MZ_OK;
     int32_t result = 0;
+    uint8_t ignore_case = UNZ_DEFAULT_IGNORE_CASE;
+
+    if (!compat)
+        return UNZ_PARAMERROR;
+
+    if (filename_case == 1) {
+        ignore_case = 0;
+    } else if (filename_case > 1) {
+        ignore_case = 1;
+    }
+
+    preserve_index = compat->entry_index;
+
+    err = mz_zip_goto_first_entry(compat->handle);
+    while (err == MZ_OK) {
+        err = mz_zip_entry_get_info(compat->handle, &file_info);
+        if (err != MZ_OK)
+            break;
+
+        result = mz_path_compare_wc(filename, file_info->filename, !ignore_case);
+
+        if (result == 0)
+            return MZ_OK;
+
+        err = mz_zip_goto_next_entry(compat->handle);
+    }
+
+    compat->entry_index = preserve_index;
+    return err;
+}
+#else
+int unzLocateFile(unzFile file, const char* filename, unzFileNameComparer filename_compare_func) {
+    mz_compat* compat = (mz_compat*)file;
+    mz_zip_file* file_info = NULL;
+    uint64_t preserve_index = 0;
+    int32_t err = MZ_OK;
+    int32_t result = 0;
 
     if (!compat)
         return UNZ_PARAMERROR;
@@ -1101,7 +1171,8 @@ int unzLocateFile(unzFile file, const char *filename, unzFileNameComparer filena
 
         if ((intptr_t)filename_compare_func > 2) {
             result = filename_compare_func(file, filename, file_info->filename);
-        } else {
+        }
+        else {
             int32_t case_sensitive = (int32_t)(intptr_t)filename_compare_func;
             result = mz_path_compare_wc(filename, file_info->filename, !case_sensitive);
         }
@@ -1115,6 +1186,7 @@ int unzLocateFile(unzFile file, const char *filename, unzFileNameComparer filena
     compat->entry_index = preserve_index;
     return err;
 }
+#endif
 
 /***************************************************************************/
 
